-------------------------------------------------------------------------------- 
-- UPC - EETAC - EEL - CSD: Digital Circuits and Systems - DIGSYS         
--------------------------------------------------------------------------------
-- LAB5 example tutorial on prototyping: asynchronous circuit 
-- Target board: DE10-Lite Terasic 
-- Target PLD: MAX10 10M50DAF484C7

-- Plan C2: FSM and some additional components for:
-- Adapting buttons and LED
-- Generating CLK signal from board external oscillator or from a push-button
-- 7-segment decoding for displaying the output codes
-- Two codes can be selected: BCD or HEX

-- This file is the translation of the reference schematic at: 
-- https://digsys.upc.edu/csd/P05/lab05/lab5.html#prototyping
--------------------------------------------------------------------------------

LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;
			
ENTITY Circuit_async_top IS
	Port (
		OSC_CLK_in  		: IN 	std_logic; -- Crystal oscillator  
		CD_L				: IN	STD_LOGIC; -- async reset (CD)
		CP_L        		: IN	STD_LOGIC; -- single CLK pulse 
		Sel_CLK, Sel_freq	: IN	STD_LOGIC; -- CLK selectors
		Sel_code 			: IN	STD_LOGIC; -- Output code selection  		
		A_L, B_L, C_L, D_L	: OUT	STD_LOGIC; -- HEX0 digit for BCD units or 
		E_L, F_L, G_L		: OUT	STD_LOGIC; -- HEX symbols 
		TS_L1, TS_L2		: OUT	STD_LOGIC; -- HEX1 segments for Tens
		SEG_L 				: OUT	STD_LOGIC_VECTOR (15 DOWNTO 0);
		LED        			: OUT	STD_LOGIC_VECTOR (3 DOWNTO 0);
	
		---- All LED not used connected to '0'
		LED4, LED5, LED6, LED7, LED8, LED9 :	OUT	STD_LOGIC
   );

END Circuit_async_top;

	-- The description of the proposed schematic
	-- 
ARCHITECTURE schematic OF Circuit_async_top IS

	COMPONENT Circuit_async IS -- This is the LAB5 module
		PORT (
			CLK, CD	: IN 	STD_LOGIC; 
			Q	: OUT  	STD_LOGIC_VECTOR(3 DOWNTO 0)    
			);
	END COMPONENT;

	COMPONENT CLK_Generator IS  
      PORT(	CD,CE			: 	IN  std_logic;
			OSC_CLK_in		: 	IN  std_logic; -- 50 MHz
			CLK_A_SQ		:	OUT std_logic;  -- 200 Hz
			CLK_B_SQ		:	OUT std_logic;  -- 10 Hz
			CLK_C_SQ		:	OUT std_logic	--	1 Hz	
          );
	end COMPONENT;       
	
	COMPONENT HEX_7seg_DECODER IS  
	PORT (
		X_IN			        			: IN STD_LOGIC_VECTOR(3 DOWNTO 0); 
		BI_L                   	    		: IN STD_LOGIC;      
		A_L, B_L, C_L, D_L, E_L, F_L, G_L  	: OUT STD_LOGIC
		);
	end COMPONENT;      	

	COMPONENT MUX_2 IS  
		PORT (
			Ch0, Ch1	: 	IN 	STD_LOGIC;  -- 1 bits per channel
			S			:	IN 	STD_LOGIC;	-- to select 2 channels
			E_L			: 	IN 	STD_LOGIC;
			Y			: 	OUT	STD_LOGIC
		  );
	end COMPONENT; 

	COMPONENT Quad_MUX_2 IS  
		PORT (
			Ch0, Ch1	: 	IN 	STD_LOGIC_VECTOR(3 downto 0);  -- 4 bits per channel
			S			:	IN 	STD_LOGIC;	-- to select 2 channels
			E_L			: 	IN 	STD_LOGIC;
			Y			: 	OUT	STD_LOGIC_VECTOR(3 downto 0)
		  );
	end COMPONENT; 

	COMPONENT Bin_BCD_6bit IS  
		port (
			E_L		: 	in STD_LOGIC;
			B		: 	in STD_LOGIC_VECTOR(5 DOWNTO 0); 
			T,U		: 	out STD_LOGIC_VECTOR(3 DOWNTO 0)
		);
	end COMPONENT; 	

	COMPONENT Dec_4_16 IS   
		PORT (
			D	: IN 	STD_LOGIC_VECTOR(3 DOWNTO 0);
			E	: IN  	STD_LOGIC;
			Y_L	: OUT  	STD_LOGIC_VECTOR(15 DOWNTO 0)    
		);
	END COMPONENT;

	COMPONENT debouncing_filter IS   
      Port (	CLK 	: IN	STD_LOGIC;
				PB_L	: IN	STD_LOGIC;
				CD   	: IN 	STD_LOGIC;
				QA 		: OUT	STD_LOGIC;
				QB		: OUT	STD_LOGIC
	);
	END COMPONENT;



-- Signals for wiring the components 
	SIGNAL CD						: std_logic;
	SIGNAL CLK_OSC,CLK_pulse, CLK	: std_logic;
	Signal CLK_200Hz_SQ				: STD_LOGIC;	
	Signal CLK_10Hz_SQ, CLK_1Hz_SQ	: STD_LOGIC;
	Signal K, J, W					: STD_LOGIC_VECTOR(3 downto 0); 
	Signal G, TS, QA					: STD_LOGIC; 
	Signal P						: STD_LOGIC_VECTOR(2 downto 0);

-- Instantiation of components
	BEGIN  
	Chip1 : Circuit_async        
-- 	from component name => to signal or port name	
		PORT MAP (
			CLK		=> CLK,
			CD		=> CD,
			Q		=> K
		);
   
	Chip2 : CLK_Generator        
-- 	from component name => to signal or port name	
		PORT MAP (
			CD				=> CD,
			CE				=> '1',
			OSC_CLK_in		=> OSC_CLK_in,
			CLK_A_SQ		=> CLK_200Hz_SQ,
			CLK_B_SQ		=> CLK_10Hz_SQ,			
			CLK_C_SQ		=> CLK_1Hz_SQ
		);          

	Chip3 : MUX_2
-- 	from component name => to signal or port name	
		PORT MAP (
			Ch0		=> CLK_1Hz_SQ,
			Ch1		=> CLK_10Hz_SQ,
			S		=> Sel_freq,
			E_L		=> '0',			
			Y		=> CLK_OSC
		);          

	Chip4 : MUX_2
-- 	from component name => to signal or port name	
		PORT MAP (
			Ch0		=> CLK_OSC,
			Ch1		=> CLK_pulse,
			S		=> Sel_CLK,
			E_L		=> '0',			
			Y		=> CLK
		);          

	Chip5 : Dec_4_16
-- 	from component name => to signal or port name	
		PORT MAP (
			D		=> K,
			E		=> '1',			
			Y_L		=> SEG_L
		);          

	Chip6 : Bin_BCD_6bit  -- (used on as Bin_BCD_4bit)
-- 	from component name => to signal or port name	
		PORT MAP (
			E_L				=> '0',
			B(5) 			=> '0',
			B(4) 			=> '0',			
			B(3 downto 0)	=> K,			
			T(3 downto 1) 	=> P,
			T(0) 			=> G,			
			U				=> W		
		);          


	Chip7 : Quad_MUX_2
-- 	from component name => to signal or port name	
		PORT MAP (
			Ch0		=> K,
			Ch1		=> W,
			S		=> Sel_code,
			E_L		=> '0',			
			Y		=> J
		);  

	Chip8 : HEX_7seg_decoder        
-- 	from component name => to signal or port name	
	PORT MAP (
		X_IN			=> J,			
		BI_L			=> '1',
		A_L				=> A_L,		
		B_L				=> B_L,	
		C_L				=> C_L,	
		D_L				=> D_L,	
		E_L				=> E_L,	
		F_L				=> F_L,	
		G_L				=> G_L		
	);          

	Chip9 : debouncing_filter        
-- 	from component name => to signal or port name	
	PORT MAP (
		CLK			=> CLK_200Hz_SQ,			
		PB_L		=> CP_L,
		CD			=> CD,		
		QA			=> QA,	
		QB			=> CLK_pulse 	
	); 

 
-- logic gates and buffers to complete the circuit 
   
CD <= not (CD_L); -- The board has active-low push-buttons
-- CLK_pulse <= not(CP_L);
TS <= G and Sel_code;
TS_L1 <= not(TS);  -- Tens segments in HEX1 digit
TS_L2 <= not(TS);

LED <= K; -- The binary radix-2 code represented in 4 LED

-- All unused LED connected to '0' to blank them
LED4 <= '0';
LED5 <= '0';
LED6 <= '0';
LED7 <= '0';
LED8 <= '0';
LED9 <= '0';

END schematic;






